@@ -25,7 +25,7 @@ from oslo_db.sqlalchemy import models
 from oslo_utils import timeutils
 from sqlalchemy import and_, func, select
 from sqlalchemy import bindparam
-from sqlalchemy import Column, Integer, String, Text, schema
+from sqlalchemy import Column, Integer, String, Text, schema, Index
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import ForeignKey, DateTime, Boolean, UniqueConstraint
 from sqlalchemy.orm import backref, column_property, relationship, validates
@@ -65,6 +65,7 @@ class Service(BASE, CinderBase):
 
     __tablename__ = 'services'
     id = Column(Integer, primary_key=True)
+    uuid = Column(String(36), nullable=True, index=True)
     cluster_name = Column(String(255), nullable=True)
     host = Column(String(255))  # , ForeignKey('hosts.id'))
     binary = Column(String(255))
@@ -90,7 +91,7 @@ class Service(BASE, CinderBase):
 
     # replication_status can be: enabled, disabled, not-capable, error,
     # failed-over or not-configured
-    replication_status = Column(String(255), default="not-capable")
+    replication_status = Column(String(36), default="not-capable")
     active_backend_id = Column(String(255))
     frozen = Column(Boolean, nullable=False, default=False)
 
@@ -111,7 +112,8 @@ class Cluster(BASE, CinderBase):
     # change this field to the same value as the id which will be unique and
     # will not conflict with the creation of another cluster with the same
     # name.
-    __table_args__ = (UniqueConstraint('name', 'binary', 'race_preventer'),)
+    __table_args__ = (UniqueConstraint('name', 'binary', 'race_preventer'),
+                      CinderBase.__table_args__)
 
     id = Column(Integer, primary_key=True)
     # NOTE(geguileo): Name is constructed in the same way that Server.host but
@@ -122,6 +124,10 @@ class Cluster(BASE, CinderBase):
     disabled_reason = Column(String(255))
     race_preventer = Column(Integer, nullable=False, default=0)
 
+    replication_status = Column(String(36), default="not-capable")
+    active_backend_id = Column(String(255))
+    frozen = Column(Boolean, nullable=False, default=False)
+
     # Last heartbeat reported by any of the services of this cluster.  This is
     # not deferred since we always want to load this field.
     last_heartbeat = column_property(
@@ -189,13 +195,15 @@ class Group(BASE, CinderBase):
     group_snapshot_id = Column(String(36))
     source_group_id = Column(String(36))
 
+    replication_status = Column(String(255))
+
 
 class Cgsnapshot(BASE, CinderBase):
     """Represents a cgsnapshot."""
     __tablename__ = 'cgsnapshots'
     id = Column(String(36), primary_key=True)
 
-    consistencygroup_id = Column(String(36))
+    consistencygroup_id = Column(String(36), index=True)
     user_id = Column(String(255), nullable=False)
     project_id = Column(String(255), nullable=False)
 
@@ -215,7 +223,7 @@ class GroupSnapshot(BASE, CinderBase):
     __tablename__ = 'group_snapshots'
     id = Column(String(36), primary_key=True)
 
-    group_id = Column(String(36), nullable=False)
+    group_id = Column(String(36), nullable=False, index=True)
     user_id = Column(String(255))
     project_id = Column(String(255))
 
@@ -234,6 +242,10 @@ class GroupSnapshot(BASE, CinderBase):
 class Volume(BASE, CinderBase):
     """Represents a block storage device that can be attached to a vm."""
     __tablename__ = 'volumes'
+    __table_args__ = (Index('volumes_service_uuid_idx',
+                            'deleted', 'service_uuid'),
+                      CinderBase.__table_args__)
+
     id = Column(String(36), primary_key=True)
     _name_id = Column(String(36))  # Don't access/modify this directly!
 
@@ -279,8 +291,8 @@ class Volume(BASE, CinderBase):
     source_volid = Column(String(36))
     encryption_key_id = Column(String(36))
 
-    consistencygroup_id = Column(String(36))
-    group_id = Column(String(36))
+    consistencygroup_id = Column(String(36), index=True)
+    group_id = Column(String(36), index=True)
 
     bootable = Column(Boolean, default=False)
     multiattach = Column(Boolean, default=False)
@@ -303,6 +315,13 @@ class Volume(BASE, CinderBase):
         foreign_keys=group_id,
         primaryjoin='Volume.group_id == Group.id')
 
+    service_uuid = Column(String(36), index=True)
+    service = relationship(Service,
+                           backref="volumes",
+                           foreign_keys=service_uuid,
+                           primaryjoin='Volume.service_uuid == Service.uuid')
+    shared_targets = Column(Boolean, default=True)  # make an FK of service?
+
 
 class VolumeMetadata(BASE, CinderBase):
     """Represents a metadata key/value pair for a volume."""
@@ -310,7 +329,8 @@ class VolumeMetadata(BASE, CinderBase):
     id = Column(Integer, primary_key=True)
     key = Column(String(255))
     value = Column(String(255))
-    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False)
+    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False,
+                       index=True)
     volume = relationship(Volume, backref="volume_metadata",
                           foreign_keys=volume_id,
                           primaryjoin='and_('
@@ -324,7 +344,8 @@ class VolumeAdminMetadata(BASE, CinderBase):
     id = Column(Integer, primary_key=True)
     key = Column(String(255))
     value = Column(String(255))
-    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False)
+    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False,
+                       index=True)
     volume = relationship(Volume, backref="volume_admin_metadata",
                           foreign_keys=volume_id,
                           primaryjoin='and_('
@@ -337,7 +358,8 @@ class VolumeAttachment(BASE, CinderBase):
     __tablename__ = 'volume_attachment'
     id = Column(String(36), primary_key=True)
 
-    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False)
+    volume_id = Column(String(36), ForeignKey('volumes.id'), nullable=False,
+                       index=True)
     volume = relationship(Volume, backref="volume_attachment",
                           foreign_keys=volume_id,
                           primaryjoin='and_('
@@ -350,6 +372,9 @@ class VolumeAttachment(BASE, CinderBase):
     detach_time = Column(DateTime)
     attach_status = Column(String(255))
     attach_mode = Column(String(255))
+    connection_info = Column(Text)
+    # Stores a serialized json dict of host connector information from brick.
+    connector = Column(Text)
 
 
 class VolumeTypes(BASE, CinderBase):
@@ -360,7 +385,8 @@ class VolumeTypes(BASE, CinderBase):
     description = Column(String(255))
     # A reference to qos_specs entity
     qos_specs_id = Column(String(36),
-                          ForeignKey('quality_of_service_specs.id'))
+                          ForeignKey('quality_of_service_specs.id'),
+                          index=True)
     is_public = Column(Boolean, default=True)
     volumes = relationship(Volume,
                            backref=backref('volume_type', uselist=False),
@@ -391,10 +417,10 @@ class GroupVolumeTypeMapping(BASE, CinderBase):
     id = Column(Integer, primary_key=True, nullable=False)
     volume_type_id = Column(String(36),
                             ForeignKey('volume_types.id'),
-                            nullable=False)
+                            nullable=False, index=True)
     group_id = Column(String(36),
                       ForeignKey('groups.id'),
-                      nullable=False)
+                      nullable=False, index=True)
 
     group = relationship(
         Group,
@@ -412,9 +438,9 @@ class VolumeTypeProjects(BASE, CinderBase):
     __table_args__ = (schema.UniqueConstraint(
         "volume_type_id", "project_id", "deleted",
         name="uniq_volume_type_projects0volume_type_id0project_id0deleted"),
-    )
+        CinderBase.__table_args__)
     id = Column(Integer, primary_key=True)
-    volume_type_id = Column(Integer, ForeignKey('volume_types.id'),
+    volume_type_id = Column(String, ForeignKey('volume_types.id'),
                             nullable=False)
     project_id = Column(String(255))
     deleted = Column(Integer, default=0)
@@ -434,12 +460,11 @@ class GroupTypeProjects(BASE, CinderBase):
     __table_args__ = (schema.UniqueConstraint(
         "group_type_id", "project_id", "deleted",
         name="uniq_group_type_projects0group_type_id0project_id0deleted"),
-    )
+        CinderBase.__table_args__)
     id = Column(Integer, primary_key=True)
-    group_type_id = Column(Integer, ForeignKey('group_types.id'),
+    group_type_id = Column(String, ForeignKey('group_types.id'),
                            nullable=False)
     project_id = Column(String(255))
-    deleted = Column(Integer, default=0)
 
     group_type = relationship(
         GroupTypes,
@@ -447,7 +472,7 @@ class GroupTypeProjects(BASE, CinderBase):
         foreign_keys=group_type_id,
         primaryjoin='and_('
         'GroupTypeProjects.group_type_id == GroupTypes.id,'
-        'GroupTypeProjects.deleted == 0)')
+        'GroupTypeProjects.deleted == False)')
 
 
 class VolumeTypeExtraSpecs(BASE, CinderBase):
@@ -458,7 +483,7 @@ class VolumeTypeExtraSpecs(BASE, CinderBase):
     value = Column(String(255))
     volume_type_id = Column(String(36),
                             ForeignKey('volume_types.id'),
-                            nullable=False)
+                            nullable=False, index=True)
     volume_type = relationship(
         VolumeTypes,
         backref="extra_specs",
@@ -477,7 +502,7 @@ class GroupTypeSpecs(BASE, CinderBase):
     value = Column(String(255))
     group_type_id = Column(String(36),
                            ForeignKey('group_types.id'),
-                           nullable=False)
+                           nullable=False, index=True)
     group_type = relationship(
         GroupTypes,
         backref="group_specs",
@@ -525,7 +550,7 @@ class QualityOfServiceSpecs(BASE, CinderBase):
     """
     __tablename__ = 'quality_of_service_specs'
     id = Column(String(36), primary_key=True)
-    specs_id = Column(String(36), ForeignKey(id))
+    specs_id = Column(String(36), ForeignKey(id), index=True)
     key = Column(String(255))
     value = Column(String(255))
 
@@ -551,8 +576,8 @@ class VolumeGlanceMetadata(BASE, CinderBase):
     """Glance metadata for a bootable volume."""
     __tablename__ = 'volume_glance_metadata'
     id = Column(Integer, primary_key=True, nullable=False)
-    volume_id = Column(String(36), ForeignKey('volumes.id'))
-    snapshot_id = Column(String(36), ForeignKey('snapshots.id'))
+    volume_id = Column(String(36), ForeignKey('volumes.id'), index=True)
+    snapshot_id = Column(String(36), ForeignKey('snapshots.id'), index=True)
     key = Column(String(255))
     value = Column(Text)
     volume = relationship(Volume, backref="volume_glance_metadata",
@@ -606,7 +631,7 @@ class QuotaUsage(BASE, CinderBase):
     id = Column(Integer, primary_key=True)
 
     project_id = Column(String(255), index=True)
-    resource = Column(String(255))
+    resource = Column(String(255), index=True)
 
     in_use = Column(Integer)
     reserved = Column(Integer)
@@ -622,11 +647,19 @@ class Reservation(BASE, CinderBase):
     """Represents a resource reservation for quotas."""
 
     __tablename__ = 'reservations'
+    __table_args__ = (Index('reservations_deleted_expire_idx',
+                            'deleted', 'expire'),
+                      Index('reservations_deleted_uuid_idx',
+                            'deleted', 'uuid'),
+                      CinderBase.__table_args__)
+
     id = Column(Integer, primary_key=True)
     uuid = Column(String(36), nullable=False)
 
-    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=True)
-    allocated_id = Column(Integer, ForeignKey('quotas.id'), nullable=True)
+    usage_id = Column(Integer, ForeignKey('quota_usages.id'), nullable=True,
+                      index=True)
+    allocated_id = Column(Integer, ForeignKey('quotas.id'), nullable=True,
+                          index=True)
 
     project_id = Column(String(255), index=True)
     resource = Column(String(255))
@@ -661,9 +694,9 @@ class Snapshot(BASE, CinderBase):
     user_id = Column(String(255))
     project_id = Column(String(255))
 
-    volume_id = Column(String(36))
-    cgsnapshot_id = Column(String(36))
-    group_snapshot_id = Column(String(36))
+    volume_id = Column(String(36), index=True)
+    cgsnapshot_id = Column(String(36), index=True)
+    group_snapshot_id = Column(String(36), index=True)
     status = Column(String(255))
     progress = Column(String(255))
     volume_size = Column(Integer)
@@ -703,7 +736,7 @@ class SnapshotMetadata(BASE, CinderBase):
     value = Column(String(255))
     snapshot_id = Column(String(36),
                          ForeignKey('snapshots.id'),
-                         nullable=False)
+                         nullable=False, index=True)
     snapshot = relationship(Snapshot, backref="snapshot_metadata",
                             foreign_keys=snapshot_id,
                             primaryjoin='and_('
@@ -742,12 +775,28 @@ class Backup(BASE, CinderBase):
     snapshot_id = Column(String(36))
     data_timestamp = Column(DateTime)
     restore_volume_id = Column(String(36))
+    encryption_key_id = Column(String(36))
 
     @validates('fail_reason')
     def validate_fail_reason(self, key, fail_reason):
         return fail_reason and fail_reason[:255] or ''
 
 
+class BackupMetadata(BASE, CinderBase):
+    """Represents a metadata key/value pair for a backup."""
+    __tablename__ = 'backup_metadata'
+    id = Column(Integer, primary_key=True)
+    key = Column(String(255))
+    value = Column(String(255))
+    backup_id = Column(String(36), ForeignKey('backups.id'), nullable=False,
+                       index=True)
+    backup = relationship(Backup, backref="backup_metadata",
+                          foreign_keys=backup_id,
+                          primaryjoin='and_('
+                          'BackupMetadata.backup_id == Backup.id,'
+                          'BackupMetadata.deleted == False)')
+
+
 class Encryption(BASE, CinderBase):
     """Represents encryption requirement for a volume type.
 
@@ -776,7 +825,7 @@ class Transfer(BASE, CinderBase):
     """Represents a volume transfer request."""
     __tablename__ = 'transfers'
     id = Column(String(36), primary_key=True)
-    volume_id = Column(String(36), ForeignKey('volumes.id'))
+    volume_id = Column(String(36), ForeignKey('volumes.id'), index=True)
     display_name = Column(String(255))
     salt = Column(String(255))
     crypt_hash = Column(String(255))
@@ -793,8 +842,8 @@ class DriverInitiatorData(BASE, models.TimestampMixin, models.ModelBase):
     __tablename__ = 'driver_initiator_data'
     __table_args__ = (
         schema.UniqueConstraint("initiator", "namespace", "key"),
-        {'mysql_engine': 'InnoDB'}
-    )
+        CinderBase.__table_args__)
+
     id = Column(Integer, primary_key=True, nullable=False)
     initiator = Column(String(255), index=True, nullable=False)
     namespace = Column(String(255), nullable=False)
@@ -806,24 +855,30 @@ class Message(BASE, CinderBase):
     """Represents a message"""
     __tablename__ = 'messages'
     id = Column(String(36), primary_key=True, nullable=False)
-    project_id = Column(String(36), nullable=False)
+    project_id = Column(String(255), nullable=False)
     # Info/Error/Warning.
     message_level = Column(String(255), nullable=False)
     request_id = Column(String(255), nullable=True)
     resource_type = Column(String(255))
-    # The uuid of the related resource.
+    # The UUID of the related resource.
     resource_uuid = Column(String(36), nullable=True)
     # Operation specific event ID.
     event_id = Column(String(255), nullable=False)
+    # Message detail ID.
+    detail_id = Column(String(10), nullable=True)
+    # Operation specific action.
+    action_id = Column(String(10), nullable=True)
     # After this time the message may no longer exist
-    expires_at = Column(DateTime, nullable=True)
+    expires_at = Column(DateTime, nullable=True, index=True)
 
 
 class ImageVolumeCacheEntry(BASE, models.ModelBase):
     """Represents an image volume cache entry"""
     __tablename__ = 'image_volume_cache_entries'
+
     id = Column(Integer, primary_key=True, nullable=False)
     host = Column(String(255), index=True, nullable=False)
+    cluster_name = Column(String(255), nullable=True)
     image_id = Column(String(36), index=True, nullable=False)
     image_updated_at = Column(DateTime, nullable=False)
     volume_id = Column(String(36), nullable=False)
@@ -835,7 +890,7 @@ class Worker(BASE, CinderBase):
     """Represents all resources that are being worked on by a node."""
     __tablename__ = 'workers'
     __table_args__ = (schema.UniqueConstraint('resource_type', 'resource_id'),
-                      {'mysql_engine': 'InnoDB'})
+                      CinderBase.__table_args__)
 
     # We want to overwrite default updated_at definition so we timestamp at
     # creation as well
@@ -843,7 +898,7 @@ class Worker(BASE, CinderBase):
                         onupdate=timeutils.utcnow)
 
     # Id added for convenience and speed on some operations
-    id = Column(Integer, primary_key=True)
+    id = Column(Integer, primary_key=True, autoincrement=True)
 
     # Type of the resource we are working on (Volume, Snapshot, Backup) it must
     # match the Versioned Object class name.
@@ -855,7 +910,10 @@ class Worker(BASE, CinderBase):
     status = Column(String(255), nullable=False)
 
     # Service that is currently processing the operation
-    service_id = Column(Integer, nullable=True)
+    service_id = Column(Integer, nullable=True, index=True)
+
+    # To prevent claiming and updating races
+    race_preventer = Column(Integer, nullable=False, default=0)
 
     # This is a flag we don't need to store in the DB as it is only used when
     # we are doing the cleanup to let decorators know
@@ -866,3 +924,27 @@ class Worker(BASE, CinderBase):
         backref="workers",
         foreign_keys=service_id,
         primaryjoin='Worker.service_id == Service.id')
+
+
+class AttachmentSpecs(BASE, CinderBase):
+    """Represents attachment specs as k/v pairs for a volume_attachment.
+
+    DO NOT USE - NOTHING SHOULD WRITE NEW DATA TO THIS TABLE
+
+    The volume_attachment.connector column should be used instead.
+    """
+
+    __tablename__ = 'attachment_specs'
+    id = Column(Integer, primary_key=True)
+    key = Column(String(255))
+    value = Column(String(255))
+    attachment_id = Column(String(36), ForeignKey('volume_attachment.id'),
+                           nullable=False, index=True)
+    volume_attachment = relationship(
+        VolumeAttachment,
+        backref="attachment_specs",
+        foreign_keys=attachment_id,
+        primaryjoin='and_('
+        'AttachmentSpecs.attachment_id == VolumeAttachment.id,'
+        'AttachmentSpecs.deleted == False)'
+    )
