@@ -13,20 +13,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 """Volume driver for Kaminario K2 all-flash arrays."""
-import six
-
 from oslo_log import log as logging
 
 from cinder import coordination
 from cinder import exception
-from cinder.i18n import _, _LE
+from cinder.i18n import _
 from cinder.objects import fields
+from cinder import utils
 from cinder.volume.drivers.kaminario import kaminario_common as common
 from cinder.zonemanager import utils as fczm_utils
 
 K2_REP_FAILED_OVER = fields.ReplicationStatus.FAILED_OVER
 LOG = logging.getLogger(__name__)
-kaminario_logger = common.kaminario_logger
 
 
 class KaminarioFCDriver(common.KaminarioCinderDriver):
@@ -45,14 +43,14 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
     # ThirdPartySystems wiki page name
     CI_WIKI_NAME = "Kaminario_K2_CI"
 
-    @kaminario_logger
+    @utils.trace
     def __init__(self, *args, **kwargs):
         super(KaminarioFCDriver, self).__init__(*args, **kwargs)
         self._protocol = 'FC'
         self.lookup_service = fczm_utils.create_lookup_service()
 
-    @fczm_utils.AddFCZone
-    @kaminario_logger
+    @fczm_utils.add_fc_zone
+    @utils.trace
     @coordination.synchronized('{self.k2_lock_name}')
     def initialize_connection(self, volume, connector):
         """Attach K2 volume to host."""
@@ -84,8 +82,8 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
                          "target_wwn": target_wwpns,
                          "initiator_target_map": init_target_map}}
 
-    @fczm_utils.RemoveFCZone
-    @kaminario_logger
+    @fczm_utils.remove_fc_zone
+    @utils.trace
     @coordination.synchronized('{self.k2_lock_name}')
     def terminate_connection(self, volume, connector, **kwargs):
         # To support replication failback
@@ -126,7 +124,7 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
             raise exception.KaminarioCinderDriverException(reason=msg)
         return target_wwpns
 
-    @kaminario_logger
+    @utils.trace
     def _get_host_object(self, connector):
         host_name = self.get_initiator_host_name(connector)
         LOG.debug("Searching initiator hostname: %s in K2.", host_name)
@@ -138,10 +136,9 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
                 host = self.client.new("hosts", name=host_name,
                                        type="Linux").save()
             except Exception as ex:
-                LOG.exception(_LE("Unable to create host : %s in K2."),
+                LOG.exception("Unable to create host : %s in K2.",
                               host_name)
-                raise exception.KaminarioCinderDriverException(
-                    reason=six.text_type(ex.message))
+                raise exception.KaminarioCinderDriverException(reason=ex)
         else:
             # Use existing host.
             LOG.debug("Use existing initiator hostname: %s in K2.", host_name)
@@ -160,11 +157,10 @@ class KaminarioFCDriver(common.KaminarioCinderDriver):
                 except Exception as ex:
                     if host_rs.total == 0:
                         self._delete_host_by_name(host_name)
-                    LOG.exception(_LE("Unable to add wwpn : %(wwpn)s to "
-                                      "host: %(host)s in K2."),
+                    LOG.exception("Unable to add wwpn : %(wwpn)s to "
+                                  "host: %(host)s in K2.",
                                   {'wwpn': wwpn, 'host': host_name})
-                    raise exception.KaminarioCinderDriverException(
-                        reason=six.text_type(ex.message))
+                    raise exception.KaminarioCinderDriverException(reason=ex)
         return host, host_rs, host_name
 
     def _build_initiator_target_map(self, connector, all_target_wwns):
